@@ -25,6 +25,7 @@
 #endif
 
 #include "camel-gw-listener.h"
+#include "camel-groupwise-settings.h"
 #include <string.h>
 #include <glib/gi18n.h>
 #include <camel/camel.h>
@@ -170,20 +171,28 @@ lookup_account_info (const gchar *key)
 #define SELECTED_NOTES   "/apps/evolution/calendar/memos/selected_memos"
 
 static void
-add_esource (const gchar *conf_key, GwAccountInfo *info,  const gchar *source_name, CamelURL *url, const gchar * parent_id_name, gboolean can_create)
+add_esource (const gchar *conf_key,
+             GwAccountInfo *info,
+             const gchar *source_name,
+             CamelURL *url,
+             const gchar *parent_id_name,
+             gboolean can_create)
 {
+	CamelSettings *settings;
 	ESourceList *source_list;
 	ESourceGroup *group;
 	ESource *source;
 	GConfClient* client;
 	GSList *ids, *temp;
+	CamelNetworkSecurityMethod security_method;
 	const gchar *source_selection_key;
 	gchar *relative_uri;
-	const gchar *soap_port;
-	const gchar * use_ssl;
+	const gchar *use_ssl;
 	const gchar *poa_address;
-	const gchar *offline_sync;
 	const gchar *group_name;
+	guint16 soap_port;
+	gboolean stay_synchronized;
+	gchar *port_string;
 
 	poa_address = url->host;
 	if (!poa_address || strlen (poa_address) ==0)
@@ -191,14 +200,31 @@ add_esource (const gchar *conf_key, GwAccountInfo *info,  const gchar *source_na
 
 	group_name = info->name;
 
-	soap_port = camel_url_get_param (url, "soap_port");
+	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, url);
 
-	if (!soap_port || strlen (soap_port) == 0)
-		soap_port = "7191";
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"stay-synchronized", &stay_synchronized,
+		"security-method", &security_method,
+		NULL);
 
-	use_ssl = camel_url_get_param (url, "use_ssl");
+	g_object_unref (settings);
 
-	offline_sync = camel_url_get_param (url, "offline_sync");
+	switch (security_method) {
+		case CAMEL_NETWORK_SECURITY_METHOD_NONE:
+			use_ssl = "never";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_SSL_ON_ALTERNATE_PORT:
+			use_ssl = "always";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_STARTTLS_ON_STANDARD_PORT:
+			use_ssl = "when-possible";
+			break;
+		default:
+			g_return_if_reached ();
+	}
 
 	client = gconf_client_get_default ();
 	source_list = e_source_list_new_for_gconf (client, conf_key);
@@ -210,14 +236,18 @@ add_esource (const gchar *conf_key, GwAccountInfo *info,  const gchar *source_na
 	if (!can_create)
 		e_source_group_set_property (group, "create_source", "no");
 
+	port_string = g_strdup_printf ("%u", soap_port);
+
 	relative_uri = g_strdup_printf ("%s@%s/", url->user, poa_address);
 	source = e_source_new (source_name, relative_uri);
 	e_source_set_property (source, "auth", "1");
 	e_source_set_property (source, "username", url->user);
-	e_source_set_property (source, "port", soap_port);
+	e_source_set_property (source, "port", port_string);
 	e_source_set_property (source, "auth-domain", "Groupwise");
 	e_source_set_property (source, "use_ssl", use_ssl);
 
+	g_free (port_string);
+
 	if (info->auto_check) {
 		gchar *str = g_strdup_printf ("%d", info->auto_check_time);
 
@@ -226,7 +256,9 @@ add_esource (const gchar *conf_key, GwAccountInfo *info,  const gchar *source_na
 	} else
 		e_source_set_property (source, "refresh", NULL);
 
-	e_source_set_property (source, "offline_sync", offline_sync ? "1" : "0" );
+	e_source_set_property (
+		source, "offline_sync",
+		stay_synchronized ? "1" : "0" );
 	e_source_set_property (source, "delete", "no");
 	if (parent_id_name) {
 		e_source_set_property (source, "parent_id_name", parent_id_name);
@@ -347,9 +379,15 @@ modify_esource (const gchar * conf_key, GwAccountInfo *old_account_info, EAccoun
 	CamelURL *url;
 	gboolean found_group;
 	GConfClient* client;
+	CamelSettings *settings;
+	CamelNetworkSecurityMethod security_method;
 	const gchar *poa_address;
 	const gchar *new_poa_address;
-	const gchar * new_group_name = a->name;
+	const gchar *new_group_name = a->name;
+	const gchar *use_ssl;
+	guint16 soap_port;
+	gboolean stay_synchronized;
+	gchar *port_string;
 
 	url = camel_url_new (old_account_info->source_url, NULL);
 	poa_address = url->host;
@@ -364,6 +402,34 @@ modify_esource (const gchar * conf_key, GwAccountInfo *old_account_info, EAccoun
 
 	found_group = FALSE;
 
+	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, new_url);
+
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"stay-synchronized", &stay_synchronized,
+		"security-method", &security_method,
+		NULL);
+
+	g_object_unref (settings);
+
+	switch (security_method) {
+		case CAMEL_NETWORK_SECURITY_METHOD_NONE:
+			use_ssl = "never";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_SSL_ON_ALTERNATE_PORT:
+			use_ssl = "always";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_STARTTLS_ON_STANDARD_PORT:
+			use_ssl = "when-possible";
+			break;
+		default:
+			g_return_if_reached ();
+	}
+
+	port_string = g_strdup_printf ("%u", soap_port);
+
 	for (; groups != NULL &&  !found_group; groups = g_slist_next (groups)) {
 		ESourceGroup *group = E_SOURCE_GROUP (groups->data);
 
@@ -385,9 +451,9 @@ modify_esource (const gchar * conf_key, GwAccountInfo *old_account_info, EAccoun
 					e_source_group_set_name (group, new_group_name);
 					e_source_set_relative_uri (source, new_relative_uri);
 					e_source_set_property (source, "username", new_url->user);
-					e_source_set_property (source, "port", camel_url_get_param (new_url,"soap_port"));
-					e_source_set_property (source, "use_ssl",  camel_url_get_param (url, "use_ssl"));
-					e_source_set_property (source, "offline_sync",  camel_url_get_param (url, "offline_sync") ? "1" : "0");
+					e_source_set_property (source, "port", port_string);
+					e_source_set_property (source, "use_ssl", use_ssl);
+					e_source_set_property (source, "offline_sync", stay_synchronized ? "1" : "0");
 
 					if (a->source->auto_check) {
 						gchar *str = g_strdup_printf ("%d", a->source->auto_check_time);
@@ -409,6 +475,8 @@ modify_esource (const gchar * conf_key, GwAccountInfo *old_account_info, EAccoun
 		}
 	}
 
+	g_free (port_string);
+
 	g_object_unref (list);
 	g_object_unref (client);
 	camel_url_free (url);
@@ -461,21 +529,23 @@ remove_calendar_tasks_sources (GwAccountInfo *info)
 static GList*
 get_addressbook_names_from_server (gchar *source_url)
 {
+	CamelSettings *settings;
 	gchar *key;
 	EGwConnection *cnc;
 	gchar *password;
 	GList *book_list = NULL;
 	gint status, count = 0;
-	const gchar *soap_port;
 	CamelURL *url;
 	gboolean remember;
 	gchar *failed_auth = NULL;
 	gchar *prompt;
 	gchar *password_prompt;
 	gchar *uri;
-	const gchar *use_ssl;
 	const gchar *poa_address;
 	guint32 flags = E_PASSWORDS_REMEMBER_FOREVER|E_PASSWORDS_SECRET;
+	CamelNetworkSecurityMethod security_method;
+	const gchar *scheme;
+	guint16 soap_port;
 
 	url = camel_url_new (source_url, NULL);
 	if (url == NULL) {
@@ -485,17 +555,25 @@ get_addressbook_names_from_server (gchar *source_url)
 	if (!poa_address || strlen (poa_address) ==0)
 		return NULL;
 
-        soap_port = camel_url_get_param (url, "soap_port");
-	if (!soap_port || strlen (soap_port) == 0)
-                soap_port = "7191";
-	use_ssl = camel_url_get_param (url, "use_ssl");
+	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, url);
 
-	key =  g_strdup_printf ("groupwise://%s@%s/", url->user, poa_address);
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"security-method", &security_method,
+		NULL);
 
-	if (use_ssl && g_str_equal (use_ssl, "always"))
-		uri = g_strdup_printf ("https://%s:%s/soap", poa_address, soap_port);
+	g_object_unref (settings);
+
+	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
 	else
-		uri = g_strdup_printf ("http://%s:%s/soap", poa_address, soap_port);
+		scheme = "https";
+
+	key = g_strdup_printf ("groupwise://%s@%s/", url->user, poa_address);
+
+	uri = g_strdup_printf ("%s://%s:%u/soap", scheme, poa_address, soap_port);
 
 	cnc = NULL;
 
@@ -503,18 +581,18 @@ get_addressbook_names_from_server (gchar *source_url)
 		count++;
 		/*we have to uncache the password before prompting again*/
 		if (failed_auth) {
-			e_passwords_forget_password ("Groupwise", key);
+			e_passwords_forget_password (NULL, key);
 			password = NULL;
 		}
 
-		password = e_passwords_get_password ("Groupwise", key);
+		password = e_passwords_get_password (NULL, key);
 		if (!password) {
 			/* Translators: First %s is the server name, second %s is user name */
 			password_prompt = g_strdup_printf (_("Enter password for %s (user %s)"),
 					poa_address, url->user);
 			prompt = g_strconcat (failed_auth ? failed_auth : "", password_prompt, NULL);
 			g_free (password_prompt);
-			password = e_passwords_ask_password (prompt, "Groupwise", key, prompt,
+			password = e_passwords_ask_password (prompt, NULL, key, prompt,
 					flags, &remember,
 					NULL);
 			g_free (prompt);
@@ -576,17 +654,21 @@ add_proxy_sources (GwAccountInfo *info, const gchar *parent_name)
 static gboolean
 add_addressbook_sources (EAccount *account)
 {
+	CamelSettings *settings;
 	CamelURL *url;
 	ESourceList *list;
 	ESourceGroup *group;
 	ESource *source;
 	gchar *base_uri;
-	const gchar *soap_port;
 	GList *books_list, *temp_list;
 	GConfClient* client;
-	const gchar * use_ssl;
+	const gchar *use_ssl;
 	const gchar *poa_address;
 	gboolean is_frequent_contacts = FALSE, is_writable = FALSE;
+	CamelNetworkSecurityMethod security_method;
+	gboolean stay_synchronized;
+	gchar *port_string;
+	guint16 soap_port;
 
 	url = camel_url_new (account->source->url, NULL);
 	if (url == NULL) {
@@ -597,10 +679,32 @@ add_addressbook_sources (EAccount *account)
 	if (!poa_address || strlen (poa_address) ==0)
 		return FALSE;
 
-	soap_port = camel_url_get_param (url, "soap_port");
-	if (!soap_port || strlen (soap_port) == 0)
-		soap_port = "7191";
-	use_ssl = camel_url_get_param (url, "use_ssl");
+	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, url);
+
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"security-method", &security_method,
+		"stay-synchronized", &stay_synchronized,
+		NULL);
+
+	g_object_unref (settings);
+
+	switch (security_method) {
+		case CAMEL_NETWORK_SECURITY_METHOD_NONE:
+			use_ssl = "never";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_SSL_ON_ALTERNATE_PORT:
+			use_ssl = "always";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_STARTTLS_ON_STANDARD_PORT:
+			use_ssl = "when-possible";
+			break;
+		default:
+			g_return_val_if_reached (FALSE);
+	}
+
 	base_uri =  g_strdup_printf ("groupwise://%s@%s", url->user, poa_address);
 	client = gconf_client_get_default ();
 	list = e_source_list_new_for_gconf (client, "/apps/evolution/addressbook/sources" );
@@ -609,6 +713,9 @@ add_addressbook_sources (EAccount *account)
 	temp_list = books_list;
 	if (!temp_list)
 		return FALSE;
+
+	port_string = g_strdup_printf ("%u", soap_port);
+
 	for (; temp_list != NULL; temp_list = g_list_next (temp_list)) {
 		const gchar *book_name =  e_gw_container_get_name (E_GW_CONTAINER (temp_list->data));
 		/* is_writable is set to TRUE if the book has isPersonal property,
@@ -630,7 +737,7 @@ add_addressbook_sources (EAccount *account)
 		source = e_source_new (book_name, g_strconcat (";",book_name, NULL));
 		e_source_set_property (source, "auth", "plain/password");
 		e_source_set_property (source, "auth-domain", "Groupwise");
-		e_source_set_property (source, "port", soap_port);
+		e_source_set_property (source, "port", port_string);
 		e_source_set_property(source, "user", url->user);
 		/* mark system address book for offline usage */
 		/* FIXME: add isPersonal flag to container and use that isFrequentContact
@@ -640,8 +747,9 @@ add_addressbook_sources (EAccount *account)
 		if (!is_writable)
 			e_source_set_property (source, "offline_sync", "1");
 		else
-			e_source_set_property (source, "offline_sync",
-					       camel_url_get_param (url, "offline_sync") ? "1" : "0");
+			e_source_set_property (
+				source, "offline_sync",
+				stay_synchronized ? "1" : "0");
 		if (!is_writable)
 			e_source_set_property (source, "completion", "true");
 		if (is_frequent_contacts)
@@ -650,6 +758,9 @@ add_addressbook_sources (EAccount *account)
 		e_source_group_add_source (group, source, -1);
 		g_object_unref (source);
 	}
+
+	g_free (port_string);
+
 	e_source_list_add_group (list, group, -1);
 	e_source_list_sync (list, NULL);
 	g_object_unref (group);
@@ -666,8 +777,10 @@ add_addressbook_sources (EAccount *account)
 }
 
 static void
-modify_addressbook_sources ( EAccount *account, GwAccountInfo *existing_account_info )
+modify_addressbook_sources (EAccount *account,
+                            GwAccountInfo *existing_account_info )
 {
+	CamelSettings *settings;
 	CamelURL *url;
 	ESourceList *list;
 	ESourceGroup *group;
@@ -676,12 +789,14 @@ modify_addressbook_sources ( EAccount *account, GwAccountInfo *existing_account_
 	gboolean delete_group;
 	gchar *old_base_uri;
 	gchar *new_base_uri;
-	const gchar *soap_port;
 	const gchar *use_ssl;
 	GSList *sources;
 	ESource *source;
 	GConfClient *client;
 	const gchar *poa_address;
+	CamelNetworkSecurityMethod security_method;
+	gchar *port_string;
+	guint16 soap_port;
 
 	url = camel_url_new (existing_account_info->source_url, NULL);
 	if (url == NULL) {
@@ -701,11 +816,35 @@ modify_addressbook_sources ( EAccount *account, GwAccountInfo *existing_account_
 	poa_address = url->host;
 	if (!poa_address || strlen (poa_address) ==0)
 		return;
+
+	settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+	camel_settings_load_from_url (settings, url);
+
+	g_object_get (
+		settings,
+		"soap-port", &soap_port,
+		"security-method", &security_method,
+		NULL);
+
+	g_object_unref (settings);
+
+	switch (security_method) {
+		case CAMEL_NETWORK_SECURITY_METHOD_NONE:
+			use_ssl = "never";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_SSL_ON_ALTERNATE_PORT:
+			use_ssl = "always";
+			break;
+		case CAMEL_NETWORK_SECURITY_METHOD_STARTTLS_ON_STANDARD_PORT:
+			use_ssl = "when-possible";
+			break;
+		default:
+			g_return_if_reached ();
+	}
+
+	port_string = g_strdup_printf ("%u", soap_port);
+
 	new_base_uri = g_strdup_printf ("groupwise://%s@%s", url->user, poa_address);
-	soap_port = camel_url_get_param (url, "soap_port");
-	if (!soap_port || strlen (soap_port) == 0)
-		soap_port = "7191";
-	use_ssl = camel_url_get_param (url, "use_ssl");
 
 	client = gconf_client_get_default ();
 	list = e_source_list_new_for_gconf (client, "/apps/evolution/addressbook/sources" );
@@ -725,7 +864,7 @@ modify_addressbook_sources ( EAccount *account, GwAccountInfo *existing_account_
 				sources = e_source_group_peek_sources (group);
 				for (; sources != NULL; sources = g_slist_next (sources)) {
 					source = E_SOURCE (sources->data);
-					e_source_set_property (source, "port", soap_port);
+					e_source_set_property (source, "port", port_string);
 					e_source_set_property (source, "use_ssl", use_ssl);
 				}
 
@@ -734,6 +873,9 @@ modify_addressbook_sources ( EAccount *account, GwAccountInfo *existing_account_
 
 		}
 	}
+
+	g_free (port_string);
+
 	if (found_group && delete_group) {
 		e_source_list_remove_group (list, group);
 		e_source_list_sync (list, NULL);
@@ -746,7 +888,6 @@ modify_addressbook_sources ( EAccount *account, GwAccountInfo *existing_account_
 		g_object_unref (list);
 	camel_url_free (url);
 	g_object_unref (client);
-
 }
 
 static void
@@ -852,9 +993,7 @@ account_changed (EAccountList *account_listener, EAccount *account)
 {
 	gboolean is_gw_account;
 	CamelURL *old_url, *new_url;
-	const gchar *old_soap_port, *new_soap_port;
 	GwAccountInfo *existing_account_info;
-	const gchar *old_use_ssl, *new_use_ssl;
 	const gchar *old_poa_address, *new_poa_address;
 
 	is_gw_account = is_groupwise_account (account);
@@ -881,6 +1020,11 @@ account_changed (EAccountList *account_listener, EAccount *account)
 		g_free (existing_account_info);
 
 	} else if (existing_account_info != NULL && is_gw_account) {
+		CamelSettings *settings;
+		CamelNetworkSecurityMethod old_security_method;
+		CamelNetworkSecurityMethod new_security_method;
+		guint16 old_soap_port;
+		guint16 new_soap_port;
 
 		if (!account->enabled) {
 			account_removed (account_listener, account);
@@ -890,29 +1034,46 @@ account_changed (EAccountList *account_listener, EAccount *account)
 		/* some info of groupwise account is changed . update the sources with new info if required */
 		old_url = camel_url_new (existing_account_info->source_url, NULL);
 		old_poa_address = old_url->host;
-		old_soap_port = camel_url_get_param (old_url, "soap_port");
-		old_use_ssl = camel_url_get_param (old_url, "use_ssl");
+
+		settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+		camel_settings_load_from_url (settings, old_url);
+
+		g_object_get (
+			settings,
+			"soap-port", &old_soap_port,
+			"security-method", &old_security_method,
+			NULL);
+
+		g_object_unref (settings);
+
 		new_url = camel_url_new (account->source->url, NULL);
 		new_poa_address = new_url->host;
 
 		if (!new_poa_address || strlen (new_poa_address) ==0)
 			return;
 
-		new_soap_port = camel_url_get_param (new_url, "soap_port");
+		settings = g_object_new (CAMEL_TYPE_GROUPWISE_SETTINGS, NULL);
+		camel_settings_load_from_url (settings, new_url);
 
-		if (!new_soap_port || strlen (new_soap_port) == 0)
-			new_soap_port = "7191";
+		g_object_get (
+			settings,
+			"soap-port", &new_soap_port,
+			"security-method", &new_security_method,
+			NULL);
 
-		new_use_ssl = camel_url_get_param (new_url, "use_ssl");
+		g_object_unref (settings);
 
 		if ((old_poa_address && strcmp (old_poa_address, new_poa_address))
-		   ||  (old_soap_port && strcmp (old_soap_port, new_soap_port))
-		   ||  strcmp (old_url->user, new_url->user)
-		   || (!old_use_ssl)
-		   || strcmp (old_use_ssl, new_use_ssl)) {
+		   || old_soap_port != new_soap_port
+		   || strcmp (old_url->user, new_url->user)
+		   || old_security_method != new_security_method) {
 
 			account_removed (account_listener, account);
 			account_added (account_listener, account);
+
+			/* We freed the old account info structure and
+			 * created a new one, so fetch the new one. */
+			existing_account_info = lookup_account_info (account->uid);
 		} else if (strcmp (existing_account_info->name, account->name)) {
 
 			modify_esource ("/apps/evolution/calendar/sources", existing_account_info, account, new_url);
